import base64
import subprocess
import os


class ReplayClient():
    """
    League of Legend replay Client class
    Opens a League of Legends client and returns a process object
    """

    def __init__(self, game_dir, replay_file_dir, replay_file_name):
        self.game_dir = game_dir
        self.replay_dir = replay_file_dir
        self.replay_file_name = replay_file_name

    def run_client(self):
        process = subprocess.Popen(
            [
                self.game_dir + '\\Game\\League of Legends.exe',
                os.path.join(self.replay_dir, self.replay_file_name),
                '-GameBaseDir=' + self.game_dir,
                '-Region=KR',  # TODO set region dynamically
                '-PlatformID=KR',
                '-Locale=ko_KR',
                '-SkipBuild',
                '-EnableCrashpad=true',
                '-EnableLNP',
                '-UseDX11=1:1',
                '-UseMetal=0:1',
                '-UseNewX3D',
                '-UseNewX3DFramebuffers',
            ],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            cwd=self.game_dir + r'\Game'
        )

        return process



import serial
import time

def send_serial(port):
    py_serial = serial.Serial(
    port=port,
    baudrate=9600,
    )
    while True:
        command = 'a'
        py_serial.write(command.encode())
        time.sleep(0.1)
        if py_serial.readable():        
            response = py_serial.readline()
            print(response[:len(response)-1].decode())
            break


import base64
import os
import requests
import json
import serial

from datanashor.utils.calc_gold import calc_gold
from datanashor.metadata import MetaDataParser
from datanashor.client import ReplayClient
from datanashor.utils.send_serial import send_serial

from time import sleep

from requests.packages import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


LIVE_CLIENT_DATA_API_ROOT = 'https://127.0.0.1:2999/liveclientdata/'


class ReplayParser():
    """
    Replay Parser class for parsing League of Legends replay files (.rofl)

    replay_file_dir: Directory of League of Legends replay files. Defaults to C:/Users/username/Documents/League of Legends/Replays
    game_dir: Directory of League of Legends game files. Defaults to C:/Riot Games/League of Legends
    interval: Interval between each request to the Live Client Data API. Defaults to 0.8 seconds
    player_data_keys: Keys to parse from Live Client Data API. Defaults to ['summonerName', 'championName', 'isDead', 'level', 'scores', 'items', 'team']
    serial_port: Serial port for sending data to Arduino. Defaults to None
    delete: Delete replay files after parsing. Defaults to True
    """

    def __init__(
            self,
            replay_file_dir=os.environ['USERPROFILE'] +
        '/Documents/League of Legends/Replays',
            game_dir='C:/Riot Games/League of Legends',
            interval=0.8,
            player_data_keys=[
                'summonerName',
                'championName',
                'isDead',
                'level',
                'scores',
                'items',
                'team',
            ],
            serial_port=None,
            delete=True):

        self.replay_file_dir = replay_file_dir
        self.game_dir = game_dir
        self.interval = interval
        self.replay_files = os.listdir(replay_file_dir)
        self.metadata = MetaDataParser(replay_file_dir)
        self.player_data_keys = player_data_keys
        self.serial_port = serial_port
        self.delete = delete

    def parse(self):
        """
        Parse all replay files in the replay_file_dir directory
        """
        while True:
            print(f'Parsing {self.replay_files[0]}')

            client = ReplayClient(
                game_dir=self.game_dir,
                replay_file_dir=self.replay_file_dir,
                replay_file_name=self.replay_files[0])

            client.run_client()

            result = []
            loaded = False
            game_time_count = 0
            prev_game_time = 0

            game_meta_data = self.metadata.parse(self.replay_files[0])

            metadata_filename = 'meta' + \
                self.replay_files[0].split('.')[0] + '.json'
            with open(metadata_filename, 'w', encoding='UTF-8') as fp:
                json.dump(game_meta_data, fp, ensure_ascii=False)

            while True:
                try:
                    player_data = requests.get(LIVE_CLIENT_DATA_API_ROOT +
                                               'playerlist/', verify=False).json()
                    game_time = requests.get(
                        LIVE_CLIENT_DATA_API_ROOT + 'gamestats/',
                        verify=False).json().get('gameTime')

                    if not loaded:
                        loaded = True

                    try:
                        current_timestamp = {
                            'timestamp': game_time,
                            'player_data': [
                                {key: champ.get(key)
                                 for key in self.player_data_keys}
                                for champ in player_data
                            ]}
                        try:
                            if 0 < game_time < 30:
                                send_serial(self.serial_port)
                        except serial.serialutil.SerialException:
                            pass

                        for player in current_timestamp['player_data']:
                            player['items'] = [
                                {
                                    'itemID': item['itemID'],
                                    'count': item['count']
                                } for item in player['items']
                            ]

                            player['kills'] = player['scores']['kills']
                            player['deaths'] = player['scores']['deaths']
                            player['assists'] = player['scores']['assists']

                            del player['scores']

                        result.append(current_timestamp)
                        print('Current time: {}'.format(game_time))

                        if game_time == prev_game_time:
                            game_time_count += 1
                        else:
                            game_time_count = 0

                        if game_time_count == 3:
                            os.system(
                                "taskkill /f /im \"League of Legends.exe\"")
                            break

                        prev_game_time = game_time

                    except AttributeError:
                        print('Waiting for game to start')

                except requests.exceptions.ConnectionError:
                    if not loaded:
                        print('Waiting for League client')
                    else:
                        break
                sleep(self.interval)

            result_data_filename = 'result' + \
                self.replay_files[0].split('.')[0] + '.json'
            with open(result_data_filename, 'w', encoding='UTF-8') as file:
                json.dump(result, file, ensure_ascii=False)
            calc_gold(result_data_filename, 'item.json')

            if self.delete:
                os.remove(os.path.join(
                    self.replay_file_dir, self.replay_files[0]))
                self.replay_files = os.listdir(self.replay_file_dir)
            sleep(10)

            if not self.replay_files:
                print('There are no more replay files left to parse.')
                break

    def get_client_metadata(self):
        '''
        Returns the client metadata.
        League of Legends client must be running.
        NOTE: by client, it does not means the replay client, but the actual game client.
        '''
        try:
            raw_metadata = open(self.game_dir + '\\lockfile',
                                'r').read().split(':')
            metadata = {
                "port": raw_metadata[2],
                "token": str(base64.b64encode(f'riot:{raw_metadata[3]}'.encode('utf-8')), encoding='utf-8'),
            }

            return metadata
        except FileNotFoundError:
            raise FileNotFoundError(
                'lockfile not found. Is the client running?')


def main():
    parser = ReplayParser()
    parser.parse()


if __name__ == '__main__':
    main()



import json
import os


class MetaDataParser():
    """
    MetaDataParser class for parsing the metadata of a .rofl file
    """

    def __init__(self, replay_file_dir):
        self.replay_file_dir = replay_file_dir

    def get_raw_metadata(self, replay_file_name):
        # TODO dynamically set replay file name

        with open(os.path.join(self.replay_file_dir, replay_file_name), "rb") as file_obj:
            file_obj.seek(262)
            length_field_buffer = file_obj.read(26)
            metadata_offset = length_field_buffer[6:10]
            metadata_length = length_field_buffer[10:14]

            metadata_offset = int.from_bytes(
                metadata_offset, byteorder='little')
            metadata_length = int.from_bytes(
                metadata_length, byteorder='little')

            file_obj.seek(metadata_offset)
            replay_metadata = file_obj.read(metadata_length)
            replay_metadata = json.loads(
                str(replay_metadata, encoding="utf-8"))
            player_stats = json.loads(replay_metadata["statsJson"])

            return player_stats

    def parse(self, replay_file_name, *args):
        """
        Parse the metadata of a single .rofl file
        returns (game length, player stats)

        player stat key defaults to
        ['NAME', 'SKIN', 'TEAM', INDIVIDUAL_POSITION', 'WIN']
        """
        player_stats = self.get_raw_metadata(replay_file_name)
        metadata_keys = ['NAME', 'SKIN', 'TEAM',
                         'INDIVIDUAL_POSITION', 'WIN'] if not args else args

        result = [{key: stat.get(key)
                  for key in metadata_keys} for stat in player_stats]

        return result



import json
import requests
from bs4 import BeautifulSoup


def patch_version_parser():
    url = 'https://developer.riotgames.com/docs/lol#data-dragon_data-assets'
    response = requests.get(url)

    if response.status_code == 200:
        html = response.text
        soup = BeautifulSoup(html, 'html.parser')
        version = soup.select_one(
            'body > div.container > div.layout > div.content-container > div.content > p:nth-child(69) > code:nth-child(1)')
    else:
        raise Exception('서버 접속 불가')

    return version.get_text()


def item_patch_parser(dest_dir):
    try:
        patch_num = patch_version_parser()
    except:
        print('버전 파싱 에러')
        return

    url = requests.get(
        f'https://ddragon.leagueoflegends.com/cdn/{patch_num}/data/ko_KR/item.json')
    text = url.text
    src = json.loads(text)
    item_dict = dict()
    data = src['data']

    for item_id in data.keys():
        item_dict[item_id] = {'name': data[item_id]
                              ['name'], 'gold': data[item_id]['gold']['total']}

    with open(dest_dir, 'w', encoding='UTF-8') as dest:
        json.dump(item_dict, dest, ensure_ascii=False)



import json
from datanashor.utils.item_patch_parser import item_patch_parser


def calc_gold(replay_data_dir, item_data_dir):
    with open(replay_data_dir, encoding='UTF-8') as src:
        source = json.load(src)
    try:
        with open(item_data_dir, encoding='UTF-8') as src:
            item_dict = json.load(src)
    except FileNotFoundError:
        item_patch_parser(item_data_dir)
        with open(item_data_dir, encoding='UTF-8') as src:
            item_dict = json.load(src)

    gold = 0
    count = 0

    for data in source:
        for user in data['player_data']:
            gold = 0
            count = 1
            for item in user['items']:
                count = item['count']
                gold += (item_dict[str(item['itemID'])]['gold']) * count
            user['gold'] = gold

    with open(replay_data_dir, 'w', encoding='UTF-8') as src:
        json.dump(source, src, ensure_ascii=False)



import setuptools


with open('README.md', 'r') as readme:
    long_description = readme.read()

setuptools.setup(
    name='datanashor',
    version='0.3.2',
    author='James Jung',
    author_email='therealjamesjung@gmail.com',
    description='League of Legends live replay parser',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/fifteen-GG/DataNashor-Parser',
    license='MIT',
    packages=setuptools.find_packages(),
    keyword=['League of Legends', 'Replay', 'Parser', 'League',
             'Lol', 'Lolreplay', 'Lolreplayparser', 'Lolreplayparser', 'DataNashor', 'datanashor'],
    install_requires=['requests', 'beautifulsoup4', 'pyserial'],
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
    ],
    python_requires='>=3.9',
)
